@@ -568,6 +568,10 @@ class Module(LocalsDictNodeNG):
                     inferred.append(inferred_node.value)
         return inferred
 
+    def _public_names(self):
+        """Get the list of the names which are publicly available in this module."""
+        return [name for name in self.keys() if not name.startswith('_')]
+
     def bool_value(self):
         return True
 
@@ -887,8 +891,11 @@ class FunctionDef(bases.Statement, Lambda):
             decoratornodes += self.decorators.nodes
         decoratornodes += self.extra_decorators
         for decnode in decoratornodes:
-            for infnode in decnode.infer():
-                result.add(infnode.qname())
+            try:
+                for infnode in decnode.infer():
+                    result.add(infnode.qname())
+            except exceptions.InferenceError:
+                continue
         return result
 
     def is_bound(self):
